"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var dateFormat = require('dateformat');
var filepath = require('filepath');
var Output_service_1 = require("./Output.service");
var fs_1 = __importDefault(require("fs"));
var ProgramService = /** @class */ (function () {
    function ProgramService() {
        this.pjson = require('../../package.json');
        this.program = require('commander');
        this.current_options = {};
        this.default_options = {};
        this.current_action = "";
    }
    ProgramService.prototype.checkEnv = function () {
        var home = filepath.home();
        var config_file = home.path + '/.esbs_config';
        if (!this.program.init && !fs_1.default.existsSync(config_file)) {
            Output_service_1.outPut.error("Missing Config file. Please run \"esbs --init\" to init your config file ");
            process.exit();
        }
    };
    ProgramService.prototype.getMergedOptions = function () {
        this.getCurrentAction();
        this.mergeOptions();
        return this.current_options;
    };
    ProgramService.prototype.mergeOptions = function () {
        this.mergeEnvOption();
        this.mergeDefaultOption();
        this.setOptionByCommand();
    };
    ProgramService.prototype.mergeEnvOption = function () {
        var _this = this;
        var homepath = filepath.home();
        var require_options = [
            "ES_HOST",
            "ES_PORT",
            "STORAGE_PATH",
            "S3_BUCKET",
            "S3_CLIENT",
            "AWS_AccessKey",
            "AWS_SecretKey",
            "AWS_REGION",
            "role_arn",
            "DEBUG"
        ];
        require('dotenv').config({ path: homepath + "/.esbs_config" });
        require_options.map(function (option) {
            if (process.env[option]) {
                _this.current_options[option] = process.env[option];
            }
        });
    };
    ProgramService.prototype.mergeDefaultOption = function () {
        if (["backup"].indexOf(this.current_action) < 0) {
            return false;
        }
        var default_options = require('../../default_options.json');
        var now = new Date();
        var date_string = dateFormat(now, "yyyymmdd");
        default_options.snapshot = "snapshot-" + date_string;
        default_options.s3_dir = default_options.repository;
        default_options.location = this.current_options.STORAGE_PATH + "/" + default_options.repository;
        this.default_options = default_options;
    };
    ProgramService.prototype.setProgramOptions = function () {
        this.program
            .version(this.pjson.version)
            .option('-R, --restore', 'restore model')
            .option('-B, --backup', 'backup model')
            .option('-D, --delete', 'delete repository/snapshot')
            .option('-r, --repository <repository>', 'Set a repository name')
            .option('-s, --snapshot <snapshot>', 'Set a snapshot name')
            .option('-t, --type <type>', 'Set a storage type fs/s3 ')
            .option('-i, --indices <indices>', 'A list index default all [usage] "index1,index2"')
            .option('-d, --dir <dir>', 'set backup dir')
            .option('-w, --watch ', 'Watching action status in some cases. ')
            .option('--init', 'Create config File')
            // .option('--st, --showTasks', 'show runing tasks')
            // .option('--showIndeics', 'show nodes indices')
            .option('--sr2, --showRecovery', 'show restore status')
            .option('--sr, --showRepository', 'show Repositories')
            .option('--cr, --createRepository', 'Create Repository')
            .option('--ss, --showSnapshot', 'show Snapshots')
            .option('--aws, --isAws', 'Is the es belong to aws ?')
            .option('--ssi, --showSnapshotInfo', 'show Snapshot detail info')
            .option('--sss, --showSnapshotStatus', 'Show Current running snapshots')
            .option('--rp, --rename_pattern <rename_pattern>', 'Set rename_pattern when retore')
            .option('--rr, --rename_replacement <rename_replacement>', 'Set rename_replacement when retore')
            .parse(process.argv);
    };
    ProgramService.prototype.setOptionByCommand = function () {
        var options = [
            "repository", "type", "waitForCompletion",
            "indices", "snapshot", "location", "s3_dir",
            "watch", "rename_pattern", "rename_replacement", "isAws"
        ];
        if (["backup"].indexOf(this.current_action) > -1) {
            for (var _i = 0, options_1 = options; _i < options_1.length; _i++) {
                var option = options_1[_i];
                this.current_options[option] = this.program[option] ? this.program[option] : this.default_options[option];
            }
        }
        else {
            for (var _a = 0, options_2 = options; _a < options_2.length; _a++) {
                var option = options_2[_a];
                this.current_options[option] = this.program[option];
            }
        }
        if (this.current_options.isAws) {
            this.current_options.type = "s3";
        }
        if (!this.current_options.indices) {
            this.current_options.indices = "*";
        }
    };
    ProgramService.prototype.getCurrentAction = function () {
        if (this.program.backup) {
            this.current_action = "backup";
        }
        if (this.program.restore) {
            this.current_action = "restore";
        }
        if (this.program.delete) {
            this.current_action = "delete";
        }
        if (this.program.init) {
            this.current_action = "init";
        }
        if (this.program.showTasks) {
            this.current_action = "showTasks";
        }
        if (this.program.showIndeics) {
            this.current_action = "showIndeics";
        }
        if (this.program.showRepository) {
            this.current_action = "showRepository";
        }
        if (this.program.showSnapshot) {
            this.current_action = "showSnapshot";
        }
        if (this.program.showSnapshotInfo) {
            this.current_action = "showSnapshotInfo";
        }
        if (this.program.showSnapshotStatus) {
            this.current_action = "showSnapshotStatus";
        }
        if (this.program.showRecovery) {
            this.current_action = "showRecovery";
        }
        if (this.program.createRepository) {
            this.current_action = "createRepository";
        }
    };
    return ProgramService;
}());
var programService = new ProgramService();
exports.programService = programService;
