import { EsService as EsService2 } from "./Es2.service"
import { EsService as EsService1 } from "./Es.service"

import fs from 'fs';
import { outPut } from "./Output.service";

const filepath = require('filepath');

class ActionService {
    public es: any
    public current_options: any
    public current_action: string = ""

    setOption(options: any) {
        this.current_options = options
    }

    setAction(action: string) {
        this.current_action = action
    }

    initEsService() {
        if (this.current_action != "init") {
            if (this.current_options.isAws) {
                this.es = new EsService2(this.current_options)
            } else {
                this.es = new EsService1(this.current_options)
            }
        }
    }

    doCurrentAction() {
        if ("" == this.current_action) {
            outPut.warn("Missing a valid action. Input \'esbs -h\' to show help ")
            return false;
        }

        switch (this.current_action) {
            case "backup":
                this.setBackupAction();
                break;
            case "restore":
                this.setRestoreAction();
                break;
            case "delete":
                this.setDeleteAction();
                break;
            case "init":
                this.initEnv();
                break;
            case "showTasks":
                this.setShowTaskListAction();
                break;
            case "showRepository":
                this.setShowRepositoryListAction();
                break;
            case "showSnapshot":
                this.setShowSnapshotListAction();
                break;
            case "showSnapshotInfo":
                this.setShowSnapshotStatusAction();
                break;
            case "createRepository":
                this.createRepositoryAction();
                break;
            case "showRecovery":
                this.setShowRecoveryAction();
                break;

            case "showSnapshotStatus":
                this.setShowSnapshotStatusAction();
                break;

            default:
                break;
        }
    }

    initEnv() {
        let home = filepath.home()
        let config_file = home.path + '/.esbs_config';
        let source_file = __dirname + '/../../.esbs_config_bak';

        fs.copyFile(source_file, config_file, (err: any) => {
            if (err) throw err;
            outPut.success(`Config file .esbs_config was copied to \'${home.path}\'`);
        });
    }

    getCrẹateRepositoryOption() {

        let option = {
            repository: this.current_options.repository,
            type: this.current_options.type,
            location: this.current_options.location
        }
        return option;
    }

    getCreateSnapshotOption() {

        let option = {
            repository: this.current_options.repository,
            snapshot: this.current_options.snapshot,
            waitForCompletion: this.current_options.waitForCompletion,
            indices: this.current_options.indices
        }
        return option;
    }


    createRepositoryAction() {
        this.es.createRepository(this.current_options);
    }

    async setBackupAction() {

        await this.es.createRepository(this.current_options);
        await this.es.createSnapshot(this.current_options).then(() => {
            this.es.showSnapshotStatus(this.current_options).then(() => {
                this.es.showSnapshotStatusWatching(this.current_options);
            });
        }).catch((error: any) => {
            outPut.error(error.toString());
        });
    }

    setDeleteAction() {

        if (!this.current_options.repository) {
            outPut.error("A Repository is required")
            return false;
        }

        if (!this.current_options.snapshot) {
            this.es.deleteRepository({ repository: this.current_options.repository });
        } else {
            let _option = {
                repository: this.current_options.repository,
                snapshot: this.current_options.snapshot
            }
            this.es.deleteSnapshot(_option);
        }
    }

    setRestoreAction() {

        if (!this.current_options.repository) {
            outPut.error("A Repository is required")
            return false;
        }

        if (!this.current_options.snapshot) {
            outPut.error("A Snapshot is required")
            return false;
        }


        this.es.restore(this.current_options).then((resp: any) => {
            console.log(resp);

            if (resp.status != "successed") {
                return false
            }
            let _option = { indices: this.current_options.rename_replacement }

            this.setIndicisBySnapshotInfo();
            this.es.showRecoveryWatching(_option)
        });
    }

    setShowTaskListAction() {
        this.es.showTaskListSimple();
    }

    setShowRepositoryListAction() {
        this.es.showRepository();
    }

    setShowSnapshotListAction() {
        if (!this.current_options.repository) {
            outPut.error("A Repository is required")
            return false;
        }

        this.es.showSnapshot({
            repository: this.current_options.repository
        });
    }

    setShowSnapshotInfoAction() {
        if (!this.current_options.repository) {
            outPut.error("A Repository is required")
            return false;
        }

        if (!this.current_options.snapshot) {
            outPut.error("A Snapshot is required")
            return false;
        }

        this.es.showSnapshotInfo({
            repository: this.current_options.repository,
            snapshot: this.current_options.snapshot
        });
    }

    setShowSnapshotStatusAction() {
        let _option = {
            repository: this.current_options.repository,
            snapshot: this.current_options.snapshot
        }

        if (this.current_options.watch) {
            this.es.showSnapshotStatusWatching(_option);
        } else {
            this.es.showSnapshotStatus(_option);
        }
    }

    async setIndicisBySnapshotInfo() {
        await this.es.setIndicisBySnapshotInfo(this.current_options)
            .then((resp: any) => {
                resp = this.es.respFormat(resp)
                try {
                    this.current_options.indices = resp.snapshots[0].indices.join(",")
                } catch (error) {

                }
            })
    }

    async setShowRecoveryAction() {

        if (this.current_options.repository && this.current_options.snapshot) {
            await this.setIndicisBySnapshotInfo();
        }

        if (this.current_options.watch) {
            this.es.showRecoveryWatching(this.current_options);
        } else {
            this.es.showRecovery(this.current_options);
        }

    }

}

let actionService = new ActionService();

export { actionService }