import { outPut } from "./Output.service";
const sleep = require("sleep")
const clc = require("cli-color");

class EsServiceBase {

    public host = process.env.ES_HOST
    public port = process.env.ES_PORT
    public storage_path = process.env.STORAGE_PATH
    public watch_stop = false;
    public client: any
    public tmp_status = ""
    public pb: any
    public temporay_repository: string = ""



    respFormat(resp: any) {
        return resp;
    }

    async createRepository(option: {
        repository: string,
        type: string,
        location: string,
        S3_BUCKET: string,
        isAws: boolean,
        AWS_REGION: string,
        role_arn: string
    }) {
        let parmaters: any = {}

        switch (option.type) {
            case 'fs':
                parmaters = {
                    "type": option.type,
                    "settings": {
                        "location": option.location
                    }
                }
                break;
            case 's3':
                parmaters = {
                    "type": option.type,
                    "settings": {
                        "bucket": option.S3_BUCKET,
                        "base_path": option.repository
                    }
                }

                if (option.isAws) {
                    parmaters.settings.region = option.AWS_REGION;
                    parmaters.settings.endpoint = "";
                    parmaters.settings.role_arn = option.role_arn;
                } else {
                    parmaters.settings.client = "default";
                }

                break;
            default:
                outPut.success(clc.red("Current allowed storage method fs/s3"));
                return false;
                break;
        }

        let resp = await this.client.snapshot.createRepository({
            "repository": option.repository,
            "body": parmaters
        })

        resp = this.respFormat(resp);

        if (resp.acknowledged) {
            outPut.success("Repository create successed.");
        }

    }

    deleteRepository(option: {
        repository: string
    }) {
        outPut.success(`Delete Repository '${option.repository}' start`);
        outPut.success(`waiting...`);

        this.client.snapshot.deleteRepository({
            "repository": option.repository,
        }).then((resp: any) => {
            outPut.success(`Repository delete successed`);
        }).catch((error: any) => {
            outPut.error(error.toString());
        })

    }

    deleteSnapshot(option: {
        repository: string,
        snapshot: string
    }) {

        outPut.success(`Delete Repository '${option.repository}' Snapshot '${option.snapshot}' start`);
        outPut.success(`waiting...`);

        this.client.snapshot.delete({
            "repository": option.repository,
            "snapshot": option.snapshot
        }).then((resp: any) => {
            outPut.success(`Snapshot delete successed`);
        }).catch((error: any) => {
            outPut.error(error.toString());
        })
    }

    async createSnapshot(option: {
        repository: string,
        snapshot: string,
        waitForCompletion: boolean,
        indices: string
    }) {
        let resp = await this.client.snapshot.create({
            "repository": option.repository,
            "snapshot": option.snapshot,
            "waitForCompletion": option.waitForCompletion,
            "body": {
                "indices": option.indices,
                "ignore_unavailable": true,
                "include_global_state": false
            }
        })

        resp = this.respFormat(resp);

        if (resp.accepted) {
            outPut.success("Snapshot create start.");
        }
    }

    getSnapshotInfo(option: {
        repository: string,
        snapshot: string
    }) {
        return this.client.snapshot.get({
            "repository": option.repository,
            "snapshot": option.snapshot,
            "ignoreUnavailable": true
        }).then(function (resp: any) {
            // outPut.success(this.respFormat(resp));
            // outPut.success(resp.statusCode);
        })
    }

    setIndicisBySnapshotInfo(option: {
        repository: string,
        snapshot: string
    }) {
        return this.client.snapshot.get({
            "repository": option.repository,
            "snapshot": option.snapshot,
            "ignoreUnavailable": true
        })
    }


    showSnapshotInfo(option: {
        repository: string,
        snapshot: string
    }) {
        this.client.snapshot.get({
            "repository": option.repository,
            "snapshot": option.snapshot,
            "ignoreUnavailable": true
        }).then((resp: any) => {
            outPut.success(this.respFormat(resp));
            // outPut.success(resp.statusCode);
        })
    }

    showSnapshotStatus(option: {
        repository: any,
        snapshot: any,
        simple_output: boolean
    }) {
        let _options: any = {}
        let show_only_one = false;

        if (option.repository) { _options.repository = option.repository }
        if (option.snapshot) { _options.snapshot = option.snapshot }

        if (option.repository && option.snapshot) {
            show_only_one = true;
        }

        return this.client.snapshot.status({
            "repository": option.repository,
            "snapshot": option.snapshot,
            "ignoreUnavailable": true
        }).then((resp: any) => {
            resp = this.respFormat(resp);

            if (show_only_one) {
                if (resp.snapshots.length > 0) {
                    let snapshot = resp.snapshots[0];
                    delete snapshot.shards_stats;
                    delete snapshot.stats;
                    let _indices = [];
                    for (const i in snapshot.indices) {
                        _indices.push(i)
                    }
                    snapshot.indices = _indices;
                    if (option.simple_output) {
                        let msg = ""
                        if (snapshot.state == "STARTED") {
                            msg = `${snapshot.repository} ${snapshot.snapshot} IN PROCESS`
                        } else {
                            msg = `${snapshot.repository} ${snapshot.snapshot} ${snapshot.state} `
                        }


                        this.tmp_status = snapshot.state;
                        outPut.success(msg);
                    } else {
                        outPut.success("Snapshot status:")
                        outPut.success(snapshot);
                    }

                } else {
                    outPut.success("Can't find snapshot info.")
                }
            } else {
                if (resp.snapshots.length > 0) {
                    outPut.success("Current running snapshot:")
                    outPut.table(resp.snapshots);
                } else {
                    outPut.success("No running snapshot.")
                }
            }
        }).catch((error: any) => {
            outPut.error(error.toString());
        })
    }


    showRecoveryWatching(option: {
        indices: string,
        showPb: boolean
    }) {
        if (!this.pb) {
            this.pb = outPut.ProgressBar("Total percent", 100);
        }
        option.showPb = true
        this.showRecovery(option).then(() => {
            if (this.watch_stop) {
                return false
            }
            sleep.sleep(1)
            this.showRecoveryWatching(option)
        })
    }

    showSnapshotStatusWatching(option: {
        repository: any,
        snapshot: any
        simple_output: boolean
    }) {
        option.simple_output = true;

        this.showSnapshotStatus(option).then(() => {
            if (this.tmp_status == "STARTED") {
                sleep.sleep(3)
                this.showSnapshotStatusWatching(option)
            }
        })

    }


    async showTaskList() {
        let tasks = [];
        const response = await this.client.tasks.list()

        for (const i in response.body.nodes) {
            let node = response.body.nodes[i]

            if (node.tasks) {
                for (const taskId in node.tasks) {
                    let task = node.tasks[taskId]
                    tasks.push(task)
                }
            }
        }
        // outPut.success(tasks);
        return tasks;
    }

    async showTaskListSimple() {
        let tasks = await this.showTaskList();
        tasks = tasks.filter((task: any) => { return !task.parent_task_id; })
        tasks = tasks.map((task: any) => { return task.action; })
        return tasks;
    }


    showRepository() {
        this.client.cat.repositories({
            "v": true,
            "s": "id"
        }).then((resp: any) => {
            outPut.success(this.respFormat(resp));
        })
    }

    showSnapshot(option: {
        repository: string,
        snapshot: string
    }) {
        this.client.cat.snapshots({
            "repository": option.repository,
            "v": true,
            "s": "id"
        }).then((resp: any) => {
            outPut.success(this.respFormat(resp));
        })
    }

    showRecovery(option: {
        indices: string,
        showPb: boolean | false
    }) {

        return this.client.indices.recovery({
            "index": option.indices
        }).then((resp: any) => {
            resp = this.respFormat(resp);

            let shards_cnt = 0
            let percent_cnt = 0
            let avg_percent = 0

            for (const i in resp) {
                if (!option.showPb) {
                    outPut.success(`Index:${i}`)
                }
                for (const snapshot of resp[i].shards) {
                    shards_cnt++
                    percent_cnt += (parseFloat(snapshot.index.size.percent.replace("%", "")))
                    if (!option.showPb) {
                        outPut.success(`Snapshot ${snapshot.id} ${snapshot.index.size.percent}`);
                    }
                }
            }
            if (percent_cnt > 0) {
                avg_percent = parseFloat((percent_cnt / shards_cnt).toFixed(2));
            }

            if (option.showPb) {
                this.pb.render({ completed: avg_percent, total: 100 });
            }

            if (avg_percent >= 100 && option.showPb) {
                outPut.success('');
                outPut.success(`Restore progress completed!`);
                this.watch_stop = true;
            }

        }).catch((error: any) => {
            outPut.error(error);
        })
    }


    restore(option: {
        repository: string,
        snapshot: string,
        indices: string,
        waitForCompletion: boolean,
        rename_pattern: string,
        rename_replacement: string,
    }) {

        let _option: any = {
            "repository": option.repository,
            "snapshot": option.snapshot,
            "waitForCompletion": option.waitForCompletion,
            "body": {
                "indices": option.indices,
                "ignore_unavailable": true,
                "include_global_state": false
            }
        }
        if (option.rename_pattern) {
            _option.body.rename_pattern = option.rename_pattern
        }

        if (option.rename_replacement) {
            _option.body.rename_replacement = option.rename_replacement
        }

        return this.client.snapshot.restore(_option)
            .then((resp: any) => {
                resp = this.respFormat(resp);
                console.log(resp);
                console.log(_option);


                if (resp.accepted) {
                    outPut.success("Snapshot restore start.");
                    return { status: "successed" };
                }
                return { status: "failed" };
            }).catch((err: any) => {
                console.log(err)
                // err = this.respFormat(err);
                // if (err) {
                //     outPut.error(err.error.type)
                //     outPut.error(err.error.reason)
                // } else {
                //     outPut.error(err.toString())
                // }
                return { status: "failed" };
            })
    }


}

export { EsServiceBase } 