
const dateFormat = require('dateformat');
const filepath = require('filepath');
import { outPut } from "./Output.service";


import fs from 'fs';


class ProgramService {

    public pjson = require('../../package.json');
    public program = require('commander');
    public current_options: any = {}
    public default_options: any = {}
    public current_action: string = ""

    checkEnv() {
        let home = filepath.home()
        let config_file = home.path + '/.esbs_config';

        if (!this.program.init && !fs.existsSync(config_file)) {
            outPut.error(`Missing Config file. Please run "esbs --init" to init your config file `)
            process.exit()
        }
    }

    getMergedOptions() {
        this.getCurrentAction();
        this.mergeOptions()
        return this.current_options
    }

    mergeOptions() {
        this.mergeEnvOption()
        this.mergeDefaultOption()
        this.setOptionByCommand()
    }

    mergeEnvOption() {
        let homepath = filepath.home()
        let require_options = [
            "ES_HOST",
            "ES_PORT",
            "STORAGE_PATH",
            "S3_BUCKET",
            "S3_CLIENT",
            "AWS_AccessKey",
            "AWS_SecretKey",
            "AWS_REGION",
            "role_arn",
            "DEBUG"
        ];

        require('dotenv').config({ path: homepath + "/.esbs_config" })

        require_options.map(option => {
            if (process.env[option]) {
                this.current_options[option] = process.env[option]
            }
        })
    }

    mergeDefaultOption() {
        if (["backup"].indexOf(this.current_action) < 0) { return false; }
        let default_options = require('../../default_options.json');


        let now = new Date();
        let date_string = dateFormat(now, "yyyymmdd");

        default_options.snapshot = "snapshot-" + date_string;
        default_options.s3_dir = default_options.repository;
        default_options.location = this.current_options.STORAGE_PATH + "/" + default_options.repository;

        this.default_options = default_options
    }

    setProgramOptions() {
        this.program
            .version(this.pjson.version)
            .option('-R, --restore', 'restore model')
            .option('-B, --backup', 'backup model')
            .option('-D, --delete', 'delete repository/snapshot')
            .option('-r, --repository <repository>', 'Set a repository name')
            .option('-s, --snapshot <snapshot>', 'Set a snapshot name')
            .option('-t, --type <type>', 'Set a storage type fs/s3 ')
            .option('-i, --indices <indices>', 'A list index default all [usage] "index1,index2"')
            .option('-d, --dir <dir>', 'set backup dir')
            .option('-w, --watch ', 'Watching action status in some cases. ')
            .option('--init', 'Create config File')
            // .option('--st, --showTasks', 'show runing tasks')
            // .option('--showIndeics', 'show nodes indices')
            .option('--sr2, --showRecovery', 'show restore status')
            .option('--sr, --showRepository', 'show Repositories')
            .option('--cr, --createRepository', 'Create Repository')
            .option('--ss, --showSnapshot', 'show Snapshots')
            .option('--aws, --isAws', 'Is the es belong to aws ?')
            .option('--ssi, --showSnapshotInfo', 'show Snapshot detail info')
            .option('--sss, --showSnapshotStatus', 'Show Current running snapshots')
            .option('--rp, --rename_pattern <rename_pattern>', 'Set rename_pattern when retore')
            .option('--rr, --rename_replacement <rename_replacement>', 'Set rename_replacement when retore')
            .parse(process.argv);
    }

    setOptionByCommand() {
        let options = [
            "repository", "type", "waitForCompletion",
            "indices", "snapshot", "location", "s3_dir",
            "watch", "rename_pattern", "rename_replacement", "isAws"
        ]

        if (["backup"].indexOf(this.current_action) > -1) {

            for (const option of options) {
                this.current_options[option] = this.program[option] ? this.program[option] : this.default_options[option];
            }
        } else {
            for (const option of options) {
                this.current_options[option] = this.program[option];
            }
        }

        if (this.current_options.isAws) {
            this.current_options.type = "s3"
        }

        if (!this.current_options.indices) {
            this.current_options.indices = "*"
        }
    }

    getCurrentAction() {

        if (this.program.backup) { this.current_action = "backup"; }
        if (this.program.restore) { this.current_action = "restore"; }
        if (this.program.delete) { this.current_action = "delete"; }

        if (this.program.init) { this.current_action = "init"; }
        if (this.program.showTasks) { this.current_action = "showTasks"; }
        if (this.program.showIndeics) { this.current_action = "showIndeics"; }
        if (this.program.showRepository) { this.current_action = "showRepository"; }
        if (this.program.showSnapshot) { this.current_action = "showSnapshot"; }
        if (this.program.showSnapshotInfo) { this.current_action = "showSnapshotInfo"; }
        if (this.program.showSnapshotStatus) { this.current_action = "showSnapshotStatus"; }
        if (this.program.showRecovery) { this.current_action = "showRecovery"; }

        if (this.program.createRepository) { this.current_action = "createRepository"; }

    }

}

let programService = new ProgramService();

export { programService };